/*****************************************************************************
 * Date: 20090123
 * Author: Mario Roessler, DH5YM
 * License: GNU General Public License
 * Description: Receive and Transmit HDLC/AX25 Frames
 *
 *
 *
 *****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "lcd_1.h"
#include "uart.h"
#include "kiss.h"
#include "hdlc.h"
//#include "uart_noint.h"

#define F_CPU 16000000UL  // Clock speed = 16MHz
#define XTAL 16000000

#define UART_BAUD_RATE 19200
//these dividers need to be defined manually because
//calculation does not fit to the correct speed (rounding)
//values for 20MHz clock:
#define BAUD_38400    25
#define BAUD_57600    16
#define BAUD_115200    8

/* 
** constants/macros (copied from LCD library)
*/
#define DDR(x) (*(&x - 1))      /* address of data direction register of port x */
#if defined(__AVR_ATmega64__) || defined(__AVR_ATmega128__)
    /* on ATmega64/128 PINF is on port 0x00 and not 0x60 */
    #define PIN(x) ( &PORTF==&(x) ? _SFR_IO8(0x00) : (*(&x - 2)) )
#else
	#define PIN(x) (*(&x - 2))    /* address of input register of port x          */
#endif


/***************************
** Globally needed variables
 ***************************/
static volatile uint8_t rxflag=0;



/************************
 * Main routine
 ************************/
int main (void)
{

  //char strbuf[10];
  //static volatile uint8_t rxflag=0;

  //set PD5 to output
  DDRD |= (1<<PD5 | 1<<PD6);

  /* Initialize UART */
  //uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
  uart_init(BAUD_115200);

  DDRA |= (1<<PA0); //NRZ-NRZI conversion
  DDRA |= (1<<PA1); //debug bit collector
  DDRA |= (1<<PA2); //debug KISS transmitter

  PORTA |= (1<<PA1 | 1<<PA2);

  sei(); //Enable interrupts

  init_rxhdlc(); //initiate the Receiver
  timer_init();  //init ms timer


  while (1==1) 
  {
     if ((PIN(DCD_PORT) & (1<<DCD_PIN)) == 0)  //if DCD=0 => start RX
  	 {
	    if(rxflag==0)
		{
          start_rxhdlc();
		  rxflag=1;
		  PORTD |= (1<<PD6);
        }
  	 } 

  	 else                        //if DCD=1 => stop RX
  	 {
          stop_rxhdlc();
		  rxflag=0;
  		  PORTD &= ~(1<<PD6);
  	 }
	 //hdlc_getbit();
	 #ifndef debug_hdlc
	 //hdlc_getframe();

	 if (framec) 
	 {
	  //framec=0;
	  bytes=bytes-2; //remove AX25 CRC checksum
	  write_kiss(pRXbuf,bytes);
	  framec--;
     }
	 #endif
  };

}



