/************************************************************************  
 *  File: kiss.c / code file for ax25-kiss handling
 *  Author: Mario Roessler, DH5YM
 *  Date: 2010/01/28
 *
 *
 ************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "kiss.h"
#include "uart.h"


#ifdef USE_CRC
/******************************************************
 * Routine for checking the CRC of a KISS/SMACK frame
 * algo from http://www.ir3ip.net/iw3fqg/doc/smak.htm
 * table for RMNC kiss comes from the TNN code (kiss.c)
 ******************************************************/
uint8_t kiss_crc(char *kissbuffer,uint16_t size, uint8_t decode)
{
  unsigned int  crc,frame_crc;
  unsigned int n=0;
  char crch,crcl;
  
  if (decode) {
    //get frame crc
	#ifdef SMACK
      frame_crc = (((unsigned int)kissbuffer[size]) << 8) | ((unsigned int)kissbuffer[((size)-1)]);
    #endif
	#ifdef RMNCKISS
      frame_crc = (((unsigned int)kissbuffer[size-1]) << 8) | ((unsigned int)kissbuffer[((size))]);
    #endif
	//calculate crc of the current frame
    crc = 0;
    while (n < (size)) {
	  #ifdef SMACK
        crc = ((crc >> 8) & 0xff) ^ crc_table[(crc ^ kissbuffer[n]) & 0xff];
	  #endif
	  #ifdef RMNCKISS
        crc = ((crc << 8)) ^ crc_table[(((crc >> 8)) ^ kissbuffer[n]) & 0x00ff];
	  #endif
	  n++;
    }

    if (frame_crc == crc) return 1;
    return 0;
  } else {
    n=0;
	#ifdef SMACK
	  crc=0x0000;
      crc = ((crc >> 8) & 0xff) ^ crc_table[(crc ^ 0x80) & 0xff]; //control byte = 0x80 for SMACK
	#endif
	#ifdef RMNCKISS
      crc=0xFFFF;
	  crc = ((crc << 8)) ^ crc_table[(((crc >> 8)) ^ 0x20) & 0x00ff]; //control byte = 0x20 for RMNC Kiss
    #endif
    while (n < (size)) {
	  #ifdef SMACK
      crc = ((crc >> 8) & 0xff) ^ crc_table[(crc ^ kissbuffer[n]) & 0xff];
	  #endif
	  #ifdef RMNCKISS
      crc = ((crc << 8)) ^ crc_table[(((crc >> 8)) ^ kissbuffer[n]) & 0x00ff];
	  #endif
	  n++;
    }
    //insert CRC into frame
	crcl=(char)(crc & 0x00FF);
	crch=(char)((crc>>8) & 0x00FF);
	#ifdef SMACK
	kissbuffer[size]=crcl;
	kissbuffer[size+1]=crch;
	#endif
	#ifdef RMNCKISS
    kissbuffer[size]=crch;
	kissbuffer[size+1]=crcl;
    #endif
    return 1;
  }
}
#endif

/****************************************************************
 * write a already assembled kiss frame with byte stuffing to UART
 ****************************************************************/
uint8_t write_kiss(char *kissbuffer,uint16_t bytes)
{
  uint16_t i=0,j=0;
  char buffc;

  char xbuf[MAXRXBUFFER*2+6];  //kiss frame transmit buffer (maximum twice the size of a kiss frame, thats not completely true because of start and stop of frame)
  
  PORTD |= (1<<PD5); //switch LED2 on

  //PORTA ^=(1<<PA2); //debug

  #ifdef USE_CRC
  kiss_crc(kissbuffer,bytes,0); //calculate CRC and insert CRC to frame
  bytes+=2; //increase buffer by 2
  #endif

  //copy frame from receiver buffer
  buffc=FEND;
  xbuf[j]=buffc;
  j++;
  #ifndef USE_CRC
  buffc=0x00;  //kiss without CRC
  #endif
  
  #ifdef USE_CRC
    #ifdef SMACK
      buffc=0x80; //kiss with SMACK CRC
    #endif
	#ifdef RMNCKISS
      buffc=0x20; //kiss with RMNC CRC
	#endif
  #endif
  
  xbuf[j]=buffc;
  j++;

  while(i<(bytes))
  {
    buffc=kissbuffer[i];
    if (buffc==(char)FEND) 
	{
	  buffc=FESC;
	  xbuf[j]=buffc;
	  j++;
	  buffc=TFEND;
	  xbuf[j]=buffc;
	  j++;
	} else if (buffc==(char)FESC) 
	{
	  buffc=FESC;
	  xbuf[j]=buffc;
	  j++;
	  buffc=TFESC;
	  xbuf[j]=buffc;
	  j++;
	} else
	{
	  xbuf[j]=buffc;
	  j++;
	}
	i++;
  }

  buffc=FEND;
  xbuf[j]=buffc;
  j++;

  bytes=0;

  i=0;
  while(i<j)
  {
    uart_putc(xbuf[i]);
    i++;
  }

  PORTD &= ~(1<<PD5); //switch LED2 off

  //PORTA ^=(1<<PA2); //debug

  return 1;
}
